/*
  ChppNextBestViewPlanner.h.cpp -

  Copyright 2008 Centre National de la Recherche Scientifique (CNRS)

  Authors: Alireza Nakhaei

*/

#ifndef HPP_NEXT_BEST_VIEW_PLANNER_H
#define HPP_NEXT_BEST_VIEW_PLANNER_H

#include <iostream>
#include <fstream>

#include "hppCore/hppPlanner.h"
#include "worldModelGrid3D/wmGrid3D.h"

#include "KineoKCDModel/kppKCDBox.h"
#include "KineoKCDModel/kppKCDPolyhedron.h"


/**
  \brief Main class of package hppNextBextViewPlanner
 */

class ChppNextBestViewPlanner: public ChppPlanner
{
	public:
		/**
		 * \brief Constructor.
		*/
		ChppNextBestViewPlanner();

		/**
		 * \brief Distructor.
		*/
		~ChppNextBestViewPlanner();

		/** \brief Set the attResolution. */
		ktStatus setResolution ( double inResolution );

		/** \brief Allocate the volume needed for the map. we should call the setResolution befor Allocate the map. */
		ktStatus allocateMap ( double xstart, double ystart, double zstart,double xend, double yend, double zend );

		/**
		 * \brief Initialize a Whole Body Planning problem. 
		 * @param  inDevice pointer to the bounding box of the robot.
		 * @return        KD_OK | KD_ERROR
		 */
		virtual ktStatus initializeProblem (CkwsDeviceShPtr inDevice);
				
		/** \brief set camera parameters */
		ktStatus setCameraParameters ( double inWidthAngle , double inHeightAngle,double inObservableDistance );

		/** \brief Set the maximum covering angle in a robot bounding box configuration */
		ktStatus setCoveringAngle ( double inHeightAngleMin, double inHeightAngleMax, double inWidthAngleMin, double inWidthAngleMax );
		
		/** \brief Return the next best view configuration for arriving in a goal configuration */
		std::vector<double> nextBextView ( double initX, double initY, double initTheta,double goalX, double goalY, double goalTheta );

		/** \brief Return the next  best view configuration for exploring the environment */
		std::vector<double> nextBextView ( double initX, double initY, double initTheta);

		/** \brief Return the visibility of a point in the model for searching a goal box */
		double visibility ( double inX, double inY, double inTheta, double goalX, double goalY, double goalTheta );

		/** \brief Return the visibility of a point in the model for exploring the environment */
		double visibility ( double inX, double inY, double inTheta );

		/** \brief Update range map */
		ktStatus updateRangeMap ( const std::vector<char> inRangeMap);

	
	protected:
		
		/** \brief Return the n nearest node in the to a configuration */
		std::vector<CkwsNodeShPtr> nearestNodes (  CkwsConfigShPtr config, unsigned int n );


	private:

		/** \brief Time parameters measuring th erequaired time for each process */
		struct timeval *Tps, *Tpf;
		struct timezone *Tzp;

		/** \brief  A copy of model of environment.*/
		CwmGrid3D att3DGridMap;
		
		/**
		   \if 0
		   \brief Count the number of object created and not destroyed
		*/
		static unsigned int nbObject;
};

#endif  //CTLC_WHOLE_BODY_PLANNER_H
